# Generated by Django 2.1 on 2018-11-06 11:01

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('django_celery_results', '0002_add_task_name_args_kwargs'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='taskresult',
            options={
                'ordering': ['-date_done'],
                'verbose_name': 'task result',
                'verbose_name_plural': 'task results'
            },
        ),
    ]
