# Generated by Django 2.2.6 on 2019-10-27 11:29

# this file is auto-generated so don't do flake8 on it
# flake8: noqa


from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('django_celery_results', '0006_taskresult_date_created'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='taskresult',
            name='hidden',
        ),
    ]
